-- GLOBAL --
SAM = select(2, ...) 
-- MODULE --
local _G = getfenv(0)
setmetatable(SAM, {__index = _G})
setfenv(1, SAM)
-- LOCAL --
title		= "StopAddonMessage"
folder		= title
version		= GetAddOnMetadata(folder, "X-Curse-Packaged-Version") or ""
titleFull	= title.." "..version

coreFrame = CreateFrame("Frame", nil, UIParent)
core	= LibStub("AceAddon-3.0"):NewAddon(coreFrame, title, "AceConsole-3.0", "AceHook-3.0", "AceEvent-3.0") -- 
local L = LibStub("AceLocale-3.0"):GetLocale(title, true)
P = {} --db.profile
G = {} --db.global

--~ directory	= "Interface\\Addons\\"..folder.."\\"

--Globals
local GAMEOPTIONS_MENU_						= _G.GAMEOPTIONS_MENU
local ENABLE_								= _G.ENABLE
local pairs_								= _G.pairs
local InterfaceOptionsFrame_OpenToCategory_	= _G.InterfaceOptionsFrame_OpenToCategory
local tostring_								= _G.tostring
local table_getn							= _G.table.getn
local print_								= _G.print
local GetFramesRegisteredForEvent_			= _G.GetFramesRegisteredForEvent
local table_insert							= _G.table.insert
local table_sort							= _G.table.sort
local UnitName_								= _G.UnitName
local date_									= _G.date;

-- Default Settings
defaultSettings = {profile = {}}
defaultSettings.profile.defaultOutgoingAction	= 0 --0=allow, 1=deny
defaultSettings.profile.defaultIncomingAction	= 0 --0=allow, 1=deny
defaultSettings.profile.printOutgoingMessages	= 4 --new
defaultSettings.profile.printIncomingMessages	= 0 --off
defaultSettings.profile.chatFrame				= 1 --main chat frame.

defaultSettings.global = {prefixActions	= {}, ignoredSenders = {}}

-- Echo Strings
local strDebugFrom		= "|cffffff00[%s]|r" --Yellow function name. help pinpoint where the debug msg is from.
local strWhiteBar		= "|cffffff00 || |r" -- a white bar to seperate the debug info.
local greenText			= "|cff00ff00%s|r"
local redText			= "|cffff0000%s|r"
local yellowText		= "|cffffff00%s|r"
local grayText			= "|cff7f7f7f%s|r"

local coreUI
local prefixUI
local senderUI
local newPrefixes = {}

local userName

--IgnoreMore addon stuff.
local ignoreMoreRunning = false 
local ignoredNames = {}


local regEvents = {
	"PLAYER_ENTERING_WORLD",
}

-- Ace3 Functions
function core:OnInitialize()
	db = LibStub("AceDB-3.0"):New("SAM_DB", defaultSettings)
	CoreOptionsTable.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)--save option profile or load another chars opts.
	db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")

	P = db.profile
	G = db.global
	
	core:RegisterChatCommand("sam", "MySlashProcessorFunc")

	local config = LibStub("AceConfig-3.0")
	local dialog = LibStub("AceConfigDialog-3.0")
	config:RegisterOptionsTable(title, CoreOptionsTable)
	coreUI = dialog:AddToBlizOptions(title, "SAM "..version)
	
	config:RegisterOptionsTable(title.."Prefix", PrefixOptionsTable)
	prefixUI = dialog:AddToBlizOptions(title.."Prefix", L["Prefixes"], "SAM "..version)

	config:RegisterOptionsTable(title.."Sender", SenderOptionsTable)
	senderUI = dialog:AddToBlizOptions(title.."Sender", L["Players"], "SAM "..version)
	
	BuildPrefixUI()
	BuildSenderUI()
	
	for i, event in pairs(regEvents) do 
		core:RegisterEvent(event)
	end
	
	--reset options on old version.
	if version ~= "" then
		if not G.lastVersion then
			G.prefixActions = {}
			G.lastVersion = version
		end
	end
end



function core:OnEnable()
	--make all options clickable.
	for name, data in pairs_(CoreOptionsTable.args.core.args) do 
		if name ~= "enable" then
			CoreOptionsTable.args.core.args[name].disabled = false
		end
	end
	
	if _G.IgM_SV then
--~ 		Debug("OnEnable", "IgnoreMore is running, lets ignore it's list of names.")
		ignoreMoreRunning = true
		UpdateIgnoreList()
		core:RegisterEvent("IGNORELIST_UPDATE")
	end
	
	userName = UnitName_("player")
	
	HookOnEventFrames()
	
	self:RawHook("SendAddonMessage", true)
	BuildPrefixUI()
end

------------------------------------------------------------------------------------------
function core:PLAYER_ENTERING_WORLD(event, ...)											--
-- Event fires after ADDON_LOADED and PLAYER_LOGIN. 									--
-- Most addons would have created their frames and registered CHAT_MSG_ADDON by now.	--
------------------------------------------------------------------------------------------
	HookOnEventFrames()
end

--------------------------------------------------
function core:IGNORELIST_UPDATE(event, ...)		--
-- Update our list of ignored names. 			--
-- This only fires if IgnoreMore is running.	--
--------------------------------------------------
	UpdateIgnoreList()
end

----------------------------------------------
function core:MySlashProcessorFunc(input)	--
-- /att function brings up the UI options.	--
----------------------------------------------
	InterfaceOptionsFrame_OpenToCategory_(prefixUI)
	InterfaceOptionsFrame_OpenToCategory_(coreUI)
end

------------------------------------------------------------------------------
function UpdateIgnoreList()													--
-- We cache ignored names. This is only if user's running IgnoreMore addon.	--
------------------------------------------------------------------------------
	ignoredNames = {} --reset, maybe they removed someone.
	local iName
	for i=1, GetNumIgnores() do
		iName = GetIgnoreName(i)
		if iName then--would this ever not exist?
			ignoredNames[iName] = true
		end
	end
end

------------------------------------------------------------------
function HookOnEventFrames()									--	/script SAM.HookOnEventFrames()
-- Raw hook frames so we can intercept their incoming messages.	--
------------------------------------------------------------------
	for i,frame in ipairs({GetFramesRegisteredForEvent("CHAT_MSG_ADDON")}) do
		if frame:HasScript("OnEvent") and not core:IsHooked(frame, "OnEvent") then
			core:RawHookScript(frame, "OnEvent", "OnEventHook")
		end
    end
end


----------------------------------------------------------------------
function core:OnEventHook(frame, event, ...)						--
-- Our incoming message hook. We intercept incoming messages here.	--
----------------------------------------------------------------------
	if event == "CHAT_MSG_ADDON" then
		local prefix, message, distribution, sender = ...
		if sender ~= userName then --no need to block our own messages right?
			if G.prefixActions[prefix] then
				if G.prefixActions[prefix].incomingAction == 0 then --allow
					G.prefixActions[prefix].incomingAllowed = G.prefixActions[prefix].incomingAllowed + 1
					
					PrintIncomingAddonMessage(L["Allowed"], false, ...)
				else
					G.prefixActions[prefix].incomingDenied = G.prefixActions[prefix].incomingDenied + 1
					PrintIncomingAddonMessage(L["Blocked"], false, ...)
					return
				end
			end
			
			if ignoreMoreRunning == true and ignoredNames[sender] then
				--sender's on our ignore list. Lets ignore what they have to say.
				Debug("OnEventHook", "Blocking icoming msg from ignored", sender)
				return
			end
			
			if G.ignoredSenders[sender] then
				return;
			end
		end
	end
	
	--send message to addon's event handler.
	self.hooks[frame].OnEvent(frame, event, ...)
end


--	/script SendAddonMessage("123", "blaw","WHISPER", "")

--	/script SAM.G.prefixActions = {}

--------------------------------------------------------------------------------------
function core:SendAddonMessage(...)													--
-- Our send addon message hook. We check what prefix is being sent and block it.	--
--------------------------------------------------------------------------------------
	local prefix, text, type, target = ...
	
	if G.prefixActions[prefix] then
		if G.prefixActions[prefix].outgoingAction == 0 then --allow
			G.prefixActions[prefix].outgoingAllowed = G.prefixActions[prefix].outgoingAllowed + 1
			
			PrintOutgoingAddonMessage(L["Allowed"], false, ...)
		else
			G.prefixActions[prefix].outgoingDenied = G.prefixActions[prefix].outgoingDenied + 1
			PrintOutgoingAddonMessage(L["Blocked"], false, ...)
			return
		end
	else
		newPrefixes[prefix] = true
		G.prefixActions[prefix] = {incomingDenied=0,incomingAllowed=0, outgoingDenied=0,outgoingAllowed=0}
		G.prefixActions[prefix].incomingAction = P.defaultIncomingAction
		G.prefixActions[prefix].outgoingAction = P.defaultOutgoingAction
		
		if P.defaultOutgoingAction == 0 then
			G.prefixActions[prefix].outgoingAllowed = 1

			PrintOutgoingAddonMessage(L["Allowed"], true, ...)
			
			BuildPrefixUI()
		else
			G.prefixActions[prefix].outgoingDenied = 1


			PrintOutgoingAddonMessage(L["Blocked"], true, ...)
			
			BuildPrefixUI()
			return
		end
	end
	
	if target then
		if ignoreMoreRunning == true and ignoredNames[target] then
			--target's on our ignore list. Lets not talk to them.
			return
		end
		
		if G.ignoredSenders[target] then
			return;
		end
	end

	core.hooks.SendAddonMessage(...)
end

--	/script SAM.G.prefixActions = {}


function core:OnDisable()
	--make all the options unclickable except for the enable button.
	for name, data in pairs_(CoreOptionsTable.args.core.args) do 
		if name ~= "enable" then
			CoreOptionsTable.args.core.args[name].disabled = true
		end
	end
end



----------------------------------------------------------------------
function core:OnProfileChanged(...)									--
-- User has reset proflie, so we reset addon.						--
----------------------------------------------------------------------
	-- Shut down anything left from previous settings
	self:Disable()
	-- Enable again with the new settings
	self:Enable()
end



------------------------------
function Debug(from, ...)	--	/script DA.Debug("hey","bye")
-- simple print function.	--
------------------------------
	if version ~= "" then--version's from curse, user doesn't need to see debug msgs.
		return
	end
	local tbl  = {...}
	local msg = tostring_(tbl[1])
	for i=2,table_getn(tbl) do 
		msg = msg..strWhiteBar..tostring_(tbl[i])
	end
	print_(strDebugFrom:format(from).." "..msg)
end

function PrintOutgoingAddonMessage(action, new, ...)
	local prefix, message, distribution, target  = ...
	
	if not G.prefixActions[prefix].printMsgs or G.prefixActions[prefix].printMsgs == false then
		if P.printOutgoingMessages == 0 then
			return
		elseif P.printOutgoingMessages == 2 and action ~= L["Allowed"] then
			return
		elseif P.printOutgoingMessages == 3 and action ~= L["Blocked"] then
			return
		elseif P.printOutgoingMessages == 4 and new ~= true then
			return
		end
	end
	
	local cf = _G["ChatFrame"..P.chatFrame]
	if cf then
		if distribution == "WHISPER" then
			cf:AddMessage("|cffffffff"..L["Outgoing"].." "..action.."|r> |cffff0000"..prefix.."|r || |cff00ff00"..distribution.."|r || |cff00ffff"..target.."|r || "..message,.5,.5,.5)
		else
			cf:AddMessage("|cffffffff"..L["Outgoing"].." "..action.."|r> |cffff0000"..prefix.."|r || |cff00ff00"..distribution.."|r || "..message,.5,.5,.5)
		end
	end
end

function PrintIncomingAddonMessage(action, new, ...)
	local prefix, message, distribution, sender  = ...
	
	if not G.prefixActions[prefix].printMsgs or G.prefixActions[prefix].printMsgs == false then
		if P.printIncomingMessages == 0 then
			return
		elseif P.printIncomingMessages == 2 and action ~= L["Allowed"] then
			return
		elseif P.printIncomingMessages == 3 and action ~= L["Blocked"] then
			return
		elseif P.printIncomingMessages == 4 and new ~= true then
			return
		end
	end
	
	local cf = _G["ChatFrame"..P.chatFrame]
	if cf then
		cf:AddMessage("|cffffffff"..L["Incoming"].." "..action.."|r> |cffff0000"..prefix.."|r || |cff00ff00"..distribution.."|r || |cff00ffff"..sender.."|r || "..message,.5,.5,.5)
	end
end

function AddNewSender(name)
	if not G.ignoredSenders[name] then
		G.ignoredSenders[name] = {}
	end
	G.ignoredSenders[name].when = date_("%c");
	
	BuildSenderUI();
end

function RemoveSender(name)
	G.ignoredSenders[name] = nil;
	BuildSenderUI();
	print(L["Removed |cffffff00%s|r from list."]:format(name))
end


----------------
--     UI     --
----------------

PrefixOptionsTable = {
	name = titleFull,
	type = "group",
	args = {},
}

PrefixOptionsTable = {
	name = titleFull,
	type = "group",
	args = {},
}

SenderOptionsTable = {
	name = titleFull,
	type = "group",
	args = {},
}

SenderOptionsTable.args.whatThisDo = {
	type = "description",	order = 1,
	name = L["Block messages incoming and outgoing to these players"],
}

tmpNewName = ""
SenderOptionsTable.args.inputName = {
	type = "input",	order	= 2,
	name = L["Name"],
	desc = L["Input a name to block"],
	set = function(info,val) 
		tmpNewName = val
	end,
	get = function(info) return tmpNewName end
}



SenderOptionsTable.args.addName = {
	type = "execute",	order	= 3,
	name	= L["Add Name"],
	desc	= L["Add sender name to list"],
	func = function(info) 
		if tmpNewName ~= "" then
			AddNewSender(tmpNewName)
			tmpNewName = ""
		end
	end
}

SenderOptionsTable.args.senderList = {
	type = "group",	order	= 4,
	name	= L["Names"],
	args={}
}


---------------------------

CoreOptionsTable = {
	name = titleFull,
	type = "group",
	childGroups = "tab",

	args = {
		core={
			name = GAMEOPTIONS_MENU_,
			type = "group",
			order = 1,
			args={}
		},
	},
}




CoreOptionsTable.args.core.args.enable = {
	type = "toggle",	order	= 1,
	name	= ENABLE_,
	desc	= L["Enables / Disables the addon"],
	set = function(info,val) 
		if val == true then
			core:Enable()
		else
			core:Disable()
		end
	end,
	get = function(info) return core:IsEnabled() end
}

CoreOptionsTable.args.core.args.chatFrame = {
	name = L["Chat Frame"],
	order = 2,
	desc = L["Chatframe to print messages to"],
	type = "select",
	
	values = function()
		local tbl = {}
		
		for i=1, 10 do 
			table_insert(tbl,(_G["ChatFrame"..i] and _G["ChatFrame"..i].name) or "ChatFrame"..i)
		end
	
		return tbl
		
	end,
	set = function(info,val) 
		P.chatFrame = val
	end,
	get = function(info) return P.chatFrame end
}


CoreOptionsTable.args.core.args.incomingHeader = {
	name	= L["Incoming"].." "..L["Prefixes"],
	order	= 10,
	type = "header",
}

CoreOptionsTable.args.core.args.defaultIncomingAction = {
	name = L["Default Action"],
	order = 11,
	desc = L["Default action for new incoming prefixes"],
	type = "select",
	
	values = function()
		return {[0]=greenText:format(L["Allow"]), [1]=redText:format(L["Block"])}
	end,
	set = function(info,val) 
		P.defaultIncomingAction = val
	end,
	get = function(info) return P.defaultIncomingAction end
}

CoreOptionsTable.args.core.args.printIncomingMessages = {
	name = L["Print Messages"],
	order = 12,
	desc = L["Print your incoming messages to a chat frame."],
	type = "select",
	
	values = function()
		return {
			[0] = L["Off"],
			[1] = L["All"],
			[2] = L["Allowed"],
			[3] = L["Blocked"],
--~ 			[4] = L["New"],
		}
		
	end,
	set = function(info,val) 
		P.printIncomingMessages = val
	end,
	get = function(info) return P.printIncomingMessages end
}



CoreOptionsTable.args.core.args.outgoingHeader = {
	name	= L["Outgoing"].." "..L["Prefixes"],
	order	= 20,
	type = "header",
}


CoreOptionsTable.args.core.args.defaultOutgoingAction = {
	name = L["Default Action"],
	order = 21,
	desc = L["Default action for new outgoing prefixes"],
	type = "select",
	
	values = function()
		return {[0]=greenText:format(L["Allow"]), [1]=redText:format(L["Block"])}
	end,
	set = function(info,val) 
		P.defaultOutgoingAction = val
	end,
	get = function(info) return P.defaultOutgoingAction end
}



CoreOptionsTable.args.core.args.printOutgoingMessages = {
	name = L["Print Messages"],
	order = 22,
	desc = L["Print your outgoing messages to a chat frame."],
	type = "select",
	
	values = function()
		return {
			[0] = L["Off"],
			[1] = L["All"],
			[2] = L["Allowed"],
			[3] = L["Blocked"],
			[4] = L["New"],
		}
		
	end,
	set = function(info,val) 
		P.printOutgoingMessages = val
	end,
	get = function(info) return P.printOutgoingMessages end
}


function BuildSenderUI()
	SenderOptionsTable.args.senderList.args = {} --reset
	
	local list = {}
	for name, data in pairs_(G.ignoredSenders) do 
		table_insert(list, name)
	end
	
	table_sort(list, function(a,b) 
		if(a and b) then 
			return a < b
		end 
	end)
	
	local sender, data;
	for i=1, table_getn(list) do 
		sender = list[i]
		data = G.ignoredSenders[sender];
		
		SenderOptionsTable.args.senderList.args[sender] = {
			type	= "group",
			name	= sender,
			desc	= "someName",
			order = i,
			args={}
		}
		
		SenderOptionsTable.args.senderList.args[sender].args.addedWhen = {
			type = "description",	order = 1,
			name = L["Added: "]..data.when,
		}
		
		
		SenderOptionsTable.args.senderList.args[sender].args.removeSender = {
			type = "execute",	order	= 10,
			name	= L["Remove Name"],
			desc	= L["Remove sender name from list"],
			func = function(info) 
				RemoveSender(info[2])
			end
		}
		
	end
	
end



function BuildPrefixUI()--	/script SAM.BuildPrefixUI()
	PrefixOptionsTable.args = {} --reset

	local addonPrefixes = {}
	
	local addonName
	
	--Get addonname of prefixes and insert them into a sortable table.
	for prefix, data in pairs_(G.prefixActions) do 
		addonName = knownPrefixes[prefix] or "_UNKNOWN";
		if not addonPrefixes[addonName] then
			addonPrefixes[addonName] = {};
		end
		table.insert(addonPrefixes[addonName], prefix)
	end

	--Sort the prefixes in each addon.
	for addon in pairs(addonPrefixes) do 
		table_sort(addonPrefixes[addon], function(a,b) 
			if(a and b) then 
				return a < b
			end 
		end)
	end
	
	--Sort the addon's list.
	local sortedAddons = {};
	for addon, data in pairs_(addonPrefixes) do 
		table.insert(sortedAddons, addon)
	end
	
	table_sort(sortedAddons, function(a,b) 
		if(a and b) then 
			return a < b
		end 
	end)
		
	
	local prefix--prefix
	local pColoured--coloured prefix
	
	local prefixes;--string list of prefixes.
	for a=1, table_getn(sortedAddons) do 
		addonName = sortedAddons[a]
		
		--Add prefixes to a string.
		prefixes = addonPrefixes[addonName][1]
		for p=2, table_getn(addonPrefixes[addonName]) do 
			prefixes = prefixes..", "..addonPrefixes[addonName][p]
		end
		
		--Create the addon's prefix table.
		PrefixOptionsTable.args[addonName] = {
			type = "group",	order	= a,
			name	= addonName,
			desc	= prefixes,
			args={}
		}
		
		--Add small desc.
		PrefixOptionsTable.args[addonName].args.prefixes = {
			type = "description",	order = 1,
			name = "Click the plus (+) sign",
		}
		
		--Add each prefix to the addon's sub menu.
		for p=1, table_getn(addonPrefixes[addonName]) do 
			prefix = addonPrefixes[addonName][p]
			
			--Colour the prefix based on blockage.
			if G.prefixActions[prefix].outgoingAction == 0 and G.prefixActions[prefix].incomingAction == 0 then
				pColoured = greenText:format(prefix)
			elseif G.prefixActions[prefix].outgoingAction == 1 and G.prefixActions[prefix].incomingAction == 1 then
				pColoured = redText:format(prefix)
			else
				pColoured = yellowText:format(prefix)
			end
	
			--Add gray 'New' text if prefix is new this session.
			if newPrefixes[prefix] then
				pColoured = pColoured .. " "..grayText:format(L["New"])
			end

			--Add the prefix menu
			PrefixOptionsTable.args[addonName].args[prefix] = {
				type	= "group",
				name	= pColoured,
				desc	= addonName,
				order = p,
				args={}
			}
		
			--Add pulldown menu of what to do on incoming messages.
			PrefixOptionsTable.args[addonName].args[prefix].args.incomingAction = {
				name = L["Incoming"].." "..L["Action"],
				order = 4,
				desc = L["What to do when message is received."],
				type = "select",
				
				values = function()
					return {[0]=greenText:format(L["Allow"]), [1]=redText:format(L["Block"])}
				end,
				set = function(info,val) 
					G.prefixActions[info[2]].incomingAction = val
					BuildPrefixUI()
				end,
				get = function(info) return G.prefixActions[info[2]].incomingAction end
			}
			
			--Add outgoing message pulldown menu.
			PrefixOptionsTable.args[addonName].args[prefix].args.outgoingAction = {
				name = L["Outgoing"].." "..L["Action"],
				order = 5,
				desc = L["What to do when message is sent."],
				type = "select",
				
				values = function()
					return {[0]=greenText:format(L["Allow"]), [1]=redText:format(L["Block"])}
				end,
				set = function(info,val) 
					G.prefixActions[info[2]].outgoingAction = val
					BuildPrefixUI()
				end,
				get = function(info) return G.prefixActions[info[2]].outgoingAction end
			}
	
			--Show how many times the incoming prefix has been allowed/blocked.
			PrefixOptionsTable.args[addonName].args[prefix].args.incomingCount = {
				type = "description",
				name = L["Incoming"]..": "..greenText:format((G.prefixActions[prefix].incomingAllowed or 0)).." :: "..redText:format((G.prefixActions[prefix].incomingDenied or 0)),
				order = 6,
			}
			
			--Show how many times the outgoing prefix has been allowed/blocked.
			PrefixOptionsTable.args[addonName].args[prefix].args.outgoingCount = {
				type = "description",
				name = L["Outgoing"]..": "..greenText:format((G.prefixActions[prefix].outgoingAllowed or 0)).." :: "..redText:format((G.prefixActions[prefix].outgoingDenied or 0)),
				order = 7,
			}
	
			--Add option to print this prefixes messages to chat.
			PrefixOptionsTable.args[addonName].args[prefix].args.printMsgs = {
				type = "toggle",	order	= 8,
				name	= L["Print Msg"],
				desc	= L["Print this prefix's messages to chat frame."],
				set = function(info,val) 
					G.prefixActions[info[2]].printMsgs = val
				end,
				get = function(info) return G.prefixActions[info[2]].printMsgs end
			}	

		end
	end
end